# Default script to create a mirror object

\set noalias = 1
if $argc != 3 || "$arg1" == "" || ($arg2 != x && $arg2 != y && $arg2 != z) then
    echo "USAGE:"
    echo "    source mirror.cmd <child_obj> <axis>"
    echo "where the parameters are:"
    echo "    <mirror_obj>  the name of the object root to mirror (under /obj)"
    echo "    <axis>        the mirror axis - must be one of: x, y, or z"
    echo "OUTPUT:"
    echo "    The name of the created mirror object is displayed"
    exit
endif

# create mirror object and wire it to our objects
set mirror = `run("opadd -n -v null mirror1")`
opset -e on -d off -p on $mirror
set parent_obj = `opinput( $arg1, 0 )`
if `strlen($parent_obj)` > 0 then
    opwire $parent_obj -0 $mirror 
endif
opwire $mirror -0 $arg1
opparm $mirror use_dcolor( 0 )

# create grid and non-rendering point
opcf $mirror

opadd grid grid1
opadd add point1

opparm point1 usept0( on )

opset -r on point1
opset -d on -t off -l off -s off -u off grid1

oplocate -x 0.5 -y 1.0 grid1
oplocate -x 2.5 -y 1.0 point1

opparm grid1 surftype( rowcol ) 
if $arg2 == x then
    opparm grid1 orient( yz )
    opcf ..
    opparm $mirror s( -1 1 1 )
endif
if $arg2 == y then
    opparm grid1 orient( zx )
    opcf ..
    opparm $mirror s( 1 -1 1 )
endif
if $arg2 == z then
    opparm grid1 orient( xy )
    opcf ..
    opparm $mirror s( 1 1 -1 )
endif

# position at the tip of a bone if necessary
if `strcmp("bone", optype($arg1))` == 0 then
    chadd -t $TSTART $TEND $mirror tz
    chkey -t $TSTART -F '-ch(strcat("../", strcat(opinput(".", 0), "/length")))' $mirror/tz
endif

# relayout mirrored objects
oplayout -s -y 0.2

