/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *	Mark Alexander
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada	 M5V 3E7
 *	416-504-9876
 *
 * NAME:	CHOP_FifthGlove.C ( Channel Operator Library, C++)
 *
 * COMMENTS:
 *
 *	Copies in one more channels from other ops.
 */
#include <fcntl.h>
#include <unistd.h>

#include <UT/UT_Interrupt.h>
#include <UT/UT_Serial.h>
#include <UT/UT_CPIO.h>
#include <UT/UT_SysClone.h>
#include <OP/OP_Operator.h>
#include <CH/CH_LocalVariable.h>
#include <CH/CH_Channel.h>
#include <PRM/PRM_Include.h>

#include <CHOP/PRM_ChopShared.h>
#include <CHOP/CHOP_Node.h>
#include "CHOP_FifthGlove.h"

//////////////////////////////////////////////////////////////////////////

#include <UT/UT_DSOVersion.h>
#include <OP/OP_OperatorTable.h>

Export void
newChopOperator(OP_OperatorTable *table)
{
    table->addOperator(
	new OP_Operator("fglove", "FifthGlove",
	CHOP_FifthGlove::myConstructor,
	&CHOP_FifthGlove::myTemplatePair, 0, 0,
	&CHOP_FifthGlove::myVariablePair,OP_FLAG_GENERATOR)
	    );
}

//////////////////////////////////////////////////////////////////////////


#define GLOVE_BAUD_RATE			19200
#define GLOVE_DATA_BITS			8
#define GLOVE_INFO_SIZE			32
#define GLOVE_RECORD_SIZE		9
#define GLOVE_GET_SINGLE_RECORD_CMD	0x43

#define GLOVE_HEADER_IDX	0
#define GLOVE_THUMB_IDX		1
#define GLOVE_FINGER1_IDX	2
#define GLOVE_FINGER2_IDX	3
#define GLOVE_FINGER3_IDX	4
#define GLOVE_FINGER4_IDX	5
#define GLOVE_PITCH_IDX		6
#define GLOVE_ROLL_IDX		7
#define GLOVE_CHECKSUM_IDX	8

#define PITCH_INDEX	0
#define ROLL_INDEX	1
#define FINGER0_INDEX	2
#define FINGER1_INDEX	3
#define FINGER2_INDEX	4
#define FINGER3_INDEX	5
#define FINGER4_INDEX	6

#define OTHER_PORT	4

#define GLOVE_RECORD_HEADER	0x80

static const char *serialPortName[] =
{
    "/dev/ttyd1",
    "/dev/ttyd2",
    "/dev/ttyd3",
    "/dev/ttyd4",
};
    
OP_TemplatePair CHOP_FifthGlove::myTemplatePair(
    CHOP_FifthGlove::myTemplateList, &CHOP_Node::myTemplatePair);

OP_VariablePair CHOP_FifthGlove::myVariablePair(
    CHOP_FifthGlove::myVariableList, &CHOP_Node::myVariablePair);

CHOP_SWITCHER2(8, "5 Glove", 8, "Channel");

static PRM_Name	       names[] =
{
    PRM_Name("port",		"Port"),
    PRM_Name("portname",	"Port Name"),
    PRM_Name("orient",		"Orientation"),
    PRM_Name("rollscale",	"Roll Scale"),
    PRM_Name("pitchscale",	"Pitch Scale"),
    PRM_Name("fingscale",	"Finger Scale"),
    PRM_Name("active",		"Active"),
    PRM_Name("calibrate",	"Calibrate"),
    
    PRM_Name("pname",		"Pitch Name"),
    PRM_Name("rname",		"Roll Name"),
    PRM_Name("tname",		"Thumb Name"),
    PRM_Name("f1name",		"Finger 1 Name"),
    PRM_Name("f2name",		"Finger 2 Name"),
    PRM_Name("f3name",		"Finger 3 Name"),
    PRM_Name("f4name",		"Finger 4 Name"),
};

static PRM_Default pitchName(0,  "pitch");
static PRM_Default rollName(0,   "roll");
static PRM_Default thumbName(0,  "thumb");
static PRM_Default finger1Name(0,"finger1");
static PRM_Default finger2Name(0,"finger2");
static PRM_Default finger3Name(0,"finger3");
static PRM_Default finger4Name(0,"finger4");

static PRM_Default rollDef(60.0);
static PRM_Default pitchDef(60.0);
static PRM_Default fingerDef(90.0);
static PRM_Default portDef(0, "/dev/ttyd5");

static PRM_Range angleRange(PRM_RANGE_UI, 0, PRM_RANGE_UI, 180);

static PRM_Default rangeDefaults[] =
{
    PRM_Default(-1.0),
    PRM_Default(1.0)
};

static PRM_Name	       portMenuItems[] =
{
    PRM_Name("serial1",	"Serial Port 1"),
    PRM_Name("serial2",	"Serial Port 2"),
    PRM_Name("serial3",	"Serial Port 3"),
    PRM_Name("serial4",	"Serial Port 4"),
    PRM_Name("other",	"Other Port"),
    PRM_Name(0),
};

static PRM_ChoiceList portMenu((PRM_ChoiceListType)(PRM_CHOICELIST_EXCLUSIVE | PRM_CHOICELIST_REPLACE),portMenuItems);

static PRM_Name	       orientMenuItems[] =
{
    PRM_Name("left",	"Left Handed Glove"),
    PRM_Name("right",	"Right Handed Glove"),
    PRM_Name(0),
};

static PRM_ChoiceList orientMenu((PRM_ChoiceListType)(PRM_CHOICELIST_EXCLUSIVE | PRM_CHOICELIST_REPLACE),orientMenuItems);

PRM_Template
CHOP_FifthGlove::myTemplateList[] =
{
    PRM_Template(PRM_SWITCHER,	3, &PRMswitcherName, switcher),
    
    PRM_Template(PRM_ORD,	1, &names[0], PRMzeroDefaults, &portMenu),
    PRM_Template(PRM_STRING,	1, &names[1], &portDef),
    PRM_Template(PRM_ORD,	1, &names[2], PRMoneDefaults, &orientMenu),
    PRM_Template(PRM_FLT_J,	1, &names[3], &rollDef, 0, &angleRange),
    PRM_Template(PRM_FLT_J,	1, &names[4], &pitchDef, 0, &angleRange),
    PRM_Template(PRM_FLT_J,	1, &names[5], &fingerDef, 0, &angleRange),
    PRM_Template(PRM_TOGGLE,	1, &names[6], PRMoneDefaults),
    PRM_Template(PRM_TOGGLE,	1, &names[7], PRMzeroDefaults),
    
    PRM_Template(PRM_STRING,	1, &names[8], &pitchName),
    PRM_Template(PRM_STRING,	1, &names[9], &rollName),
    PRM_Template(PRM_STRING,	1, &names[10], &thumbName),
    PRM_Template(PRM_STRING,	1, &names[11], &finger1Name),
    PRM_Template(PRM_STRING,	1, &names[12], &finger2Name),
    PRM_Template(PRM_STRING,	1, &names[13], &finger3Name),
    PRM_Template(PRM_STRING,	1, &names[14], &finger4Name),
    
    PRM_Template(PRM_FLT_J,	1, &CHOP_SampleRateName,
		 &CHOP_SampleRateDefault, 0, &CHOP_SampleRateRange ),
    PRM_Template(),
};

CH_LocalVariable
CHOP_FifthGlove::myVariableList[] = {
    { 0, 0, 0 }
};

OP_Node *
CHOP_FifthGlove::myConstructor(OP_Network	*net,
			   const char	*name,
			   OP_Operator	*op)
{
    return new CHOP_FifthGlove(net, name, op);
}

//----------------------------------------------------------------------------


CHOP_FifthGlove::CHOP_FifthGlove( OP_Network	*net,
			  const char	*name,
			  OP_Operator	*op)
	   : CHOP_Node(net, name, op)
{
    myParmBase = getParmList()->getParmIndex( names[0].getToken() );
    mySerial = 0;
    mySerialPort = 0;
    myActiveState = 0;
    myIsCalibrating = 0;

    for(int i=NUM_GLOVE_CHANNELS; i--;)
    {
	myMinCalibration[i] = 0;
	myMaxCalibration[i] = 255;
    }
}

CHOP_FifthGlove::~CHOP_FifthGlove()
{
}

OP_ERROR
CHOP_FifthGlove::cookMyChop(OP_Context &context)
{
    // add all the tracks, set start/SR/length
    setupClip(context);

    // get data from the glove & stick it in the channels.
    if(ACTIVE())
    {
	getFifthGloveData(context);
	OP_Node::flags().timeDep = 1;
    }
    else if(mySerial)
    {
	delete mySerial;
	mySerial = 0;
    }
    
    return error();
}

void
CHOP_FifthGlove::setupClip(OP_Context &context)
{
    UT_String	 name;
    float	 rate;
    CL_TrackList list;
    int		 i,j;
    int		 found;
    
    rate = SAMPLE_RATE(context.myTime);

    myClip->setSampleRate(rate);
    myClip->setTrackLength(1);
    myClip->setStart(0);
    
    PITCH_NAME(name);
    if(name.isstring())
    {
	myPitch = myClip->getTrack((const char *)name);
	if(!myPitch)
	    myPitch = myClip->addTrack((const char *)name);
	list.append(myPitch);
    }
    else
	myPitch = 0;

    ROLL_NAME(name);
    if(name.isstring())
    {
	myRoll = myClip->getTrack((const char *)name);
	if(!myRoll)
	    myRoll = myClip->addTrack((const char *)name);
	list.append(myRoll);
    }
    else
	myRoll = 0;

    THUMB_NAME(name);
    if(name.isstring())
    {
	myThumb = myClip->getTrack((const char *)name);
	if(!myThumb)
	    myThumb = myClip->addTrack((const char *)name);
	list.append(myThumb);
    }
    else
	myThumb = 0;

    FINGER1_NAME(name);
    if(name.isstring())
    {
	myFinger1 = myClip->getTrack((const char *)name);
	if(!myFinger1)
	    myFinger1 = myClip->addTrack((const char *)name);
	list.append(myFinger1);
    }
    else
	myFinger1 = 0;
    
    FINGER2_NAME(name);
    if(name.isstring())
    {
	myFinger2 = myClip->getTrack((const char *)name);
	if(!myFinger2)
	    myFinger2 = myClip->addTrack((const char *)name);
	list.append(myFinger2);
    }
    else
	myFinger2 = 0;
    
    FINGER3_NAME(name);
    if(name.isstring())
    {
	myFinger3 = myClip->getTrack((const char *)name);
	if(!myFinger3)
	    myFinger3 = myClip->addTrack((const char *)name);
	list.append(myFinger3);
    }
    else
	myFinger3 = 0;
    
    FINGER4_NAME(name);
    if(name.isstring())
    {
	myFinger4 = myClip->getTrack((const char *)name);
	if(!myFinger4)
	    myFinger4 = myClip->addTrack((const char *)name);
	list.append(myFinger4);
    }
    else
	myFinger4 = 0;

    
    for(i=myClip->getNumTracks(); i--;)
    {
	found = 0;
	for(j=list.entries(); j--;)
	    if(list(j) == myClip->getTrack(i))
	    {
		found = 1;
		break;
	    }
	
	if(!found)
	    myClip->removeTrack(i);
    }

    myClip->reorder(list);
}

int
CHOP_FifthGlove::openFifthGlovePort()
{
    char infobuf[GLOVE_INFO_SIZE];
    char command;
    int	 i;

    if(mySerial)
    	delete mySerial;
    
    mySerial = new UT_Serial(GLOVE_BAUD_RATE, GLOVE_DATA_BITS);

    if(!mySerial->open(mySerialName, O_RDWR))
    {
	cerr << "Couldn't open the port" << endl;
	myActiveState = 0;
	
	delete mySerial;
	mySerial = 0;
	
	return 0;
    }
    else
	myActiveState = 1;

    mySerial->flush();

    command = 0x41;
    if(mySerial->write(&command,1) < 0)
	cerr << "Error writing command to port " << endl;
    else
    {
	i = 8;
	sginap(1);
	do {
	    if(mySerial->read(&command, 1) <= 0)
	    {
		sginap(1);
		i--;
	    }
	}    while(command != 0x55 && i-->0);
	
// 	if(command != 0x55)
// 	    cerr << "ACK Didn't match" << endl;
    }
    
    command = 0x47;
     if(mySerial->write(&command,1) < 0)
 	cerr << "Error writing command to port " << endl;
     else
     {
	 sginap(5);
	 
	 // read the info block
	 getRecord(infobuf, GLOVE_INFO_SIZE, 0x42);
	 
	 if(infobuf[0] == 0x42 && infobuf[1] == 0x52)
	 {
	     myActiveState = 1;
	     
	     // write the getdata command....
	     command = GLOVE_GET_SINGLE_RECORD_CMD;
	     mySerial->write(&command,1);

	     return 1;
	 }
// 	 else
// 	     cerr << "INFO header mismatch " << endl;
     }
     
    myActiveState = 1;
    return 0;
}

int
CHOP_FifthGlove::getRecord(char *buffer, int recordsize, char header)
{
    int		timeout = 9;
    int		numtoread;
    int		read;
    char	chr = !header;

    // get the header char first....
    do {
	if(mySerial->read(&chr, 1) <= 0)
	{
	    sginap(1);
	    timeout-=2;
	}
    } while(chr != header && timeout-->=0);

    if(chr != header)
	return 0;
    
    // read the input
    read = 1;
    buffer[0] = header;
    
    timeout = 2;
    while(read < recordsize && timeout--)
    {
	numtoread = mySerial->read(buffer+read, recordsize);
	read += numtoread;
	
	if(read < recordsize)
	    sginap(1);
    }
    
    if(read < recordsize)
	return 0;

    return 1;
}

int
CHOP_FifthGlove::getFifthGloveRecord(float  &pitch,float &roll,float &f0,float &f1,
			   float &f2,float &f3,float &f4, float t)
{
    unsigned char inbuf[GLOVE_RECORD_SIZE];
    int		p, r, if0, if1, if2, if3, if4;
    float	fingsc;
    char	command;
    int		chk;
    float	temp;
    int		status = 0;
    int		portchanged = getSerialName();

    // dud data for now....
    p = r = 255;
    if0 = if1 = if2 = if3 = if4 = 128;

    if(portchanged || !myActiveState || !mySerial)
    {
	if(!openFifthGlovePort())
	{
	    addError(CHOP_ERROR_DEVICE_NOT_FOUND);
	    return 0;
	}
    }

    // get the  request data command

    if(getRecord((char*)inbuf, GLOVE_RECORD_SIZE, 0x80))
    {
	status = 1;
	
	if(GLOVE_RECORD_HEADER != int(inbuf[GLOVE_HEADER_IDX]))
	{
	    cerr << "Record Header doesn't match" << endl;
	    return 0;
	}

	p   = inbuf[GLOVE_PITCH_IDX];
	r   = inbuf[GLOVE_ROLL_IDX];
	if0 = inbuf[GLOVE_THUMB_IDX];
	if1 = inbuf[GLOVE_FINGER1_IDX];
	if2 = inbuf[GLOVE_FINGER2_IDX];
	if3 = inbuf[GLOVE_FINGER3_IDX];
	if4 = inbuf[GLOVE_FINGER4_IDX];
    
        // check the checksum
	chk = p ^ r ^ if0 ^ if1 ^ if2 ^ if3 ^ if4;

	if(chk != int(inbuf[GLOVE_CHECKSUM_IDX]))
	{
// 	cerr << "Checksum failed: " << chk << ", cs = "
// 	     << int(inbuf[GLOVE_CHECKSUM_IDX]) << endl;
	    return 0;
	}

	// do calibration if necessary.
	if(CALIBRATE())
	    getCalibration(p,r,if0,if1,if2,if3,if4);
	else
	    myIsCalibrating = 0;
	
        // scale to proper ranges
	pitch = PITCH_SCALE(t) * float(calibrationScale(PITCH_INDEX,p)-128)/128.0F;
	roll  = ROLL_SCALE(t) * float(calibrationScale(ROLL_INDEX,r)-128)/128.0F;
    
	fingsc = FINGER_SCALE(t);
    
	f0 = fingsc * float(calibrationScale(FINGER0_INDEX,if0)) / 255.0F;
	f1 = fingsc * float(calibrationScale(FINGER1_INDEX,if1)) / 255.0F;
	f2 = fingsc * float(calibrationScale(FINGER2_INDEX,if2)) / 255.0F;
	f3 = fingsc * float(calibrationScale(FINGER3_INDEX,if3)) / 255.0F;
	f4 = fingsc * float(calibrationScale(FINGER4_INDEX,if4)) / 255.0F;

        // leftie; reverse the fingers.
	if(!ORIENTATION())
	{
	    temp = f0;
	    f0 = f4;
	    f4 = temp;

	    temp = f1;
	    f1 = f3;
	    f3 = temp;
	}
    }
    
    // for next time...
    command = GLOVE_GET_SINGLE_RECORD_CMD;
    if(mySerial->write(&command,1) < 0)
	cerr << "Error writing command to port " << endl;
     
    return status;
}

void
CHOP_FifthGlove::getFifthGloveData(OP_Context &context)
{
    float	pitch=0.0F,roll=0.0F,f0=0.0F,f1=0.0F,f2=0.0F,f3=0.0F,f4=0.0F;

    if(!getFifthGloveRecord(pitch,roll,f0,f1,f2,f3,f4, context.myTime))
	return;

    if(myPitch)
	myPitch->getData()[0] = pitch;
    
    if(myRoll)
	myRoll->getData()[0] = roll;
    
    if(myThumb)
	myThumb->getData()[0] = f0;
    
    if(myFinger1)
	myFinger1->getData()[0] = f1;
    
    if(myFinger2)
	myFinger2->getData()[0] = f2;
    
    if(myFinger3)
	myFinger3->getData()[0] = f3;
    
    if(myFinger4)
	myFinger4->getData()[0] = f4;
}
    
OP_ERROR
CHOP_FifthGlove::save(ostream &os, const OP_SaveFlags &flags,
		      const char *pathPrefix)
{
    UT_CPIO      packet;
    UT_String	 path;
    const char  *ext;
    int		 i;
    int		 version = 1;
    int		 channels = NUM_GLOVE_CHANNELS;
    int		 binary = flags.isBinary();
    
    if (CHOP_Node::save(os,flags,pathPrefix) >=UT_ERROR_ABORT)
	return error();

    ext = binary ? ".bglove" : ".glove";
    
    path.harden(pathPrefix);
    path += getName();
    path += ext;

    packet.open(os, path);

    if(binary)
    {
	UTwrite(os, &version);
	UTwrite(os, &channels);

	for(i=0; i<channels; i++)
	{
	    UTwrite(os, &myMinCalibration[i]);
	    UTwrite(os, &myMaxCalibration[i]);
	}
    }
    else
    {
	os << version << endl;
	os << channels << endl;
	for(i=0; i<channels; i++)
	{
	    os << myMinCalibration[i] << endl;
	    os << myMaxCalibration[i] << endl;
	}
    }

    packet.close(os);
    
    return error();
}

OP_ERROR
CHOP_FifthGlove::load(istream &is, int binary,const char *extension,int fileSize,
		 const char *path)
{
    int version;
    int i,num,min,max;
    
    myIsCalibrating = CALIBRATE();


    if(!strcmp(extension,"bglove") || !strcmp(extension,"glove"))
    {
	if (binary)
	{
	    UTread(is, &version);
	}
	else
	{
	    is >> version; 
	}

	switch(version)
	{
	case 1:
	    if(binary)
	    {
		UTread(is, &num);

		for(i=0; i<num; i++)
		{
		    UTread(is, &min);
		    UTread(is, &max);

		    if(i < NUM_GLOVE_CHANNELS)
		    {
			myMinCalibration[i] = min;
			myMaxCalibration[i] = max;
		    }
		}
	    }
	    else
	    {
		is >> num;
		for(i=0; i<num; i++)
		{
		    is >> min;
		    is >> max;
		    
		    if(i < NUM_GLOVE_CHANNELS)
		    {
			myMinCalibration[i] = min;
			myMaxCalibration[i] = max;
		    }
		}		
	    }
	    break;
	    
	default:
	    cerr << "CHOP_FifthGlove: Unknown version " << version << endl;
	    break;
	}
    }
    else if (CHOP_Node::load(is, binary,extension,fileSize,path) >= UT_ERROR_ABORT)
	return error();
   
    return error();
}

void
CHOP_FifthGlove::getCalibration(int p,int r,int if0,int if1,int if2,int if3,int if4)
{
    int i;
    int current[NUM_GLOVE_CHANNELS];
    
    if(!myIsCalibrating)
    {
	// reset limits to crazy ones... (reversed min/max)
	for(i=0; i<NUM_GLOVE_CHANNELS; i++)
	{
	    myMinCalibration[i] = 255;
	    myMaxCalibration[i] = 0;
	}
	myIsCalibrating = 1;
    }

    current[PITCH_INDEX] = p;
    current[ROLL_INDEX] = r;
    current[FINGER0_INDEX] = if0;
    current[FINGER1_INDEX] = if1;
    current[FINGER2_INDEX] = if2;
    current[FINGER3_INDEX] = if3;
    current[FINGER4_INDEX] = if4;

    for(i=0; i<NUM_GLOVE_CHANNELS; i++)
    {
	if(current[i] > myMaxCalibration[i])
	    myMaxCalibration[i] = current[i];
	
	if(current[i] < myMinCalibration[i])
	    myMinCalibration[i] = current[i];

// 	cerr << "RANGE: " << myMinCalibration[i] << " -> "
// 	     << myMaxCalibration[i] << endl;
    }
}

int
CHOP_FifthGlove::calibrationScale(int index, int value)
{
    // return raw value if still calibrating.
    if(myIsCalibrating)
	return value;

    int	range = (int)(myMaxCalibration[index] - myMinCalibration[index]);

//     cerr << "Mapping to 0-255 from " << myMinCalibration[index]
// 	 << "-" << myMaxCalibration[index] << " " <<value << " = ";
    
    if(range)
	value = (int)(((value - myMinCalibration[index])*255.0F/float(range)) +
		      0.5F);

//    cerr << value << endl;

    // make sure it lies between the ranges; clamp
    if(value > 255)
	value = 255;
    if(value < 0)
	value = 0;
	
    return value;
}
    


unsigned
CHOP_FifthGlove::disableParms()
{
    unsigned changes = CHOP_Node::disableParms();

    changes += enableParm("calibrate", ACTIVE());
    changes += enableParm("portname", PORT() == OTHER_PORT);

    return changes;
}


int
CHOP_FifthGlove::getSerialName()
{
    UT_String name;
    int port = PORT();

    if(port == OTHER_PORT)
	CUSTOM_PORT(name);
    else
	name = serialPortName[port];


    if(name != mySerialName)
    {
	mySerialName.harden(name);
    	return 1;
    }    
    return 0 ;
}
