/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *	Mark Alexander
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada   M5V 3E7
 *	416-504-9876
 *
 * NAME:	CHOP_Puppet.h ( CHOP Library, C++)
 *
 * COMMENTS:
 */

#ifndef __CHOP_Puppet__
#define __CHOP_Puppet__

#include <UT/UT_RefArray.h>
#include <CHOP/CHOP_RealtimeGen.h>

#define ARG_PUPPET_ACTIVE	(myParmBase +  0)	// puppet
#define ARG_PUPPET_MODE		(myParmBase +  1)
#define ARG_PUPPET_REPORT_RATE	(myParmBase +  2)
#define ARG_PUPPET_USE_CYCLES	(myParmBase +  3)

#define ARG_PUPPET_GRAB_ZERO	(myParmBase +  4)	// calibrate
#define ARG_PUPPET_USE_ZERO	(myParmBase +  5)
#define ARG_PUPPET_WRITE_ZERO	(myParmBase +  6)

#define ARG_PUPPET_PORT		(myParmBase +  7)	// port
#define ARG_PUPPET_PORT_NAME	(myParmBase +  8)
#define ARG_PUPPET_BAUD_RATE	(myParmBase +  9)
#define ARG_PUPPET_RESET	(myParmBase + 10)

#define ARG_PUPPET_LEFT		(myParmBase + 11)
#define ARG_PUPPET_RIGHT	(myParmBase + 12)
#define ARG_PUPPET_DEFAULT	(myParmBase + 13)

#define SERIAL_BUF_SIZE		8096

class UT_Serial;
class UT_Interrupt;

class CHOP_Puppet : public CHOP_RealtimeGenerator
{
public:


    static OP_Node		*myConstructor(OP_Network*, const char *,
							    OP_Operator *);
    static OP_TemplatePair	 myTemplatePair;
    static OP_VariablePair	 myVariablePair;
    static PRM_Template          myTemplateList[];
    static CH_LocalVariable      myVariableList[];

    OP_ERROR			 cookMyChop(OP_Context &context);
    OP_ERROR			 cookMySlice(OP_Context &context,
					     int start, int end);

    virtual unsigned             disableParms();
    virtual int                  usesScope() const { return 0; }
    virtual int			 usesUnits() { return 0; }


    virtual	int usesRealtime()	{ return 0; }
    virtual	int isRealtimeCook()	{ return 1; }
    virtual	int realtimeMode();

    virtual	void realtimeStart(float t);
    virtual	void realtimeStop();

    virtual	float getRealtimeSampleRate();

    // callbacks
    static	int grabZero(void *, int, float, const PRM_Template *);
    static	int writeZero(void *, int, float, const PRM_Template *);
    static	int reset(void *, int, float, const PRM_Template *);
    static	int portChanged(void *, int,  float, const PRM_Template *);

    const char *    getTimeSliceExtension() { return ".pup"; }
    virtual void    getChannelOutputNames(UT_PtrArray<char*> &str);
    virtual ut_RealtimeData * newRealtimeDataBlock(const char *name,
						   const CL_Track *track,
						   float t);
  
   
protected:
				 CHOP_Puppet(OP_Network  *net, 
					const char *name, OP_Operator *op);
    virtual			~CHOP_Puppet();

private:

    // puppet

    int		ACTIVE()
		{ return evalInt(ARG_PUPPET_ACTIVE, 0, 0); }
    
    int		MODE()
		{ return evalInt(ARG_PUPPET_MODE, 0, 0); }
    
    int		REPORT_RATE()
		{ return evalInt(ARG_PUPPET_REPORT_RATE, 0, 0); }
    
    int		USE_CYCLES()
		{ return evalInt(ARG_PUPPET_USE_CYCLES, 0, 0); }
    
    int		USE_ZERO()
		{ return evalInt(ARG_PUPPET_USE_ZERO,0,0); }

    // port
    int		PORT()
		{ return evalInt(ARG_PUPPET_PORT,0,0); }
    
    void	PORT_NAME(UT_String &name)
		{ evalString(name, ARG_PUPPET_PORT_NAME,0,0); }

    int		BAUD_RATE()
		{ return evalInt(ARG_PUPPET_BAUD_RATE,0,0); }

    // channel
    int		LEXTEND()
		{ return evalInt(ARG_PUPPET_LEFT,  0, 0); }
    
    int		REXTEND()
		{ return evalInt(ARG_PUPPET_RIGHT, 0, 0); }
    
    float	DEFAULT(float t)
		{ return evalFloat(ARG_PUPPET_DEFAULT,0,t); }


    // serial link information
    UT_Serial	*myPuppet;

    int		 myPuppetState;
    UT_String	 myVersionString;
    int		 myActiveState;
    int		 myBaudRate;
    UT_String	 myPort;
    int		 myReportRate;
    int		 myReportMode;
    int		 myDemandSent;

    // puppet information
    int		 myNumJoints;
    int		 myVersionMajor;
    int		 myVersionMinor;
    int		 myTimeStamp;
    int		 myExpectedStamp;

    char	 mySerialBuffer[SERIAL_BUF_SIZE];
    int		 mySerialPos;
    int		 mySerialOffset;
    int		 mySerialSize;
    char	 myBuffer[1024];

    float	*myRecordData;
    int		 myAngleFormat;

    // config info
    int		 myConfigNumJoints;
    float	*myConfigInfo;
    int		 myIsConfigured;
    int		 myUseConfig;

    // interrupt
    UT_Interrupt *myBoss;

private:

    void	 grabZeroPosition();
    void	 savePreferenceConfig();
    void	 loadPreferenceConfig();

    void	 updateConnection();
    void	 updateJointTracks();
    void	 recordSingle();
    void	 recordTimeSlice(); 
    
    void	 openPort();
    void	 closePort();
    void	 flushPort();
    void	 changeBaud(int b);

    // Puppet serial packet interface code
    void	 sendPacket(char header, char *msg, int num);
    int		 receivePacket(int wait = 4, int recentangles = 0);


    // higher level stuff...
    int		 getVersion();
    int		 getAcknowledge(int acknum = 1);
    void	 sendInitialize();
    
    int		 sendFindJointsCmd();
    void	 sendTimedDataCommand();
    void	 sendSingleDataCommand();
    void	 sendGetDataCommand();
    void	 sendNoTimeStampCommand();
    void	 sendFrameResetCommand();
    void	 sendReportRateCommand();
    void	 sendAngleFormatCommand(int three);

    // returns # of joint angles received
    int		 getAngles(int joints, float *abuf, int &ts,
			   int mostrecent = 0,int first = 1);
 
    
};

class ut_PuppetData : public ut_RealtimeData
{
public:
	     ut_PuppetData(const char *name)
		 : ut_RealtimeData(name), myLastValue(0.0F)  { ; }
    virtual ~ut_PuppetData() { ; } 
    
    float    myLastValue;
};


#endif
