/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada   M5V 3E7
 *	416-504-9876
 *
 * NAME:	SHOP_CustomClerk.h ( SHOP Library, C++)
 *
 * COMMENTS:	This is a sample clerk interface to SHOPs.
 */

#include <UT/UT_DSOVersion.h>
#include <OP/OP_Operator.h>
#include <SHOP/SHOP_Data.h>
#include <SHOP/SHOP_Node.h>
#include "SHOP_CustomClerk.h"

SHOP_CustomClerk::SHOP_CustomClerk()
{
}

SHOP_CustomClerk::~SHOP_CustomClerk()
{
}

const char *
SHOP_CustomClerk::getRenderName() const
{
    return "CustomRender";	// This is the same name as in the IFD class
}

const char *
SHOP_CustomClerk::getRenderLabel() const
{
    return "A custom renderer";
}


int
SHOP_CustomClerk::generatesString() const
{
    return 1;		// The data we return is a single string
}

unsigned
SHOP_CustomClerk::disableParms(SHOP_Node &node)
{
    // If you choose, you can disable parms based on the value of other parms
    // at this point.
    return SHOP_Clerk::disableParms(node);
}


SHOP_Data *
SHOP_CustomClerk::buildRenderData(SHOP_Node *node, float now)
{
    PRM_Template	*list;		// Parameters of SHOP
    SHOP_StringData	*shopdata;	// Where we store the data
    UT_String		 shader, token, tmpstr;
    char		 tmpbuf[1024];
    int			 i, j, start, end, vector_size;

    // Get the parameter template for the operator
    list = node->getOperator()->getParmTemplates();

    // Ask the SHOP which parameters are "interesting"
    start = node->getFirstParameter();
    end   = node->getLastParameter();

    // Now, verify that the template is at least this long
    for (i = 0; i < start; i++)
	if (list[i].getType() == PRM_LIST_TERMINATOR)
	    return 0;

    // Set the shader name based on the internal operator name of the SHOP
    shader = node->getOperator()->getName();

    for (i = start; i < end; i++)
    {
	if (list[i].getType() == PRM_LIST_TERMINATOR)
	    break;

	// Here, we want to skip any switcher parameters
	if ((list[i].getType() & PRM_TYPE_BASIC_TYPE) == PRM_TYPE_ORDINAL &&
	    (list[i].getType() & PRM_TYPE_SWITCHERLIST))
	{
	    continue;
	}

	// Lookup the internal name of the parameter
	list[i].getToken(token);
	shader += " ";
	shader += token;	// Add to the shader string
	shader += "{";

	vector_size = list[i].getVectorSize();
	// Now, build the values for the parameters
	switch (list[i].getType() & PRM_TYPE_BASIC_TYPE)
	{
	    case PRM_TYPE_ORDINAL:
		for (j = 0; j < vector_size; j++)
		{
		    sprintf(tmpbuf, "%d", node->evalInt(i, j, now));
		    shader += " ";
		    shader += tmpbuf;
		}
		break;
	    case PRM_TYPE_STRING:
		for (j = 0; j < vector_size; j++)
		{
		    node->evalString(tmpstr, i, j, now);
		    shader += " ";
		    shader += tmpstr;
		}
		break;
	    default:
		for (j = 0; j < vector_size; j++)
		{
		    sprintf(tmpbuf, "%g", node->evalFloat(i, j, now));
		    shader += " ";
		    shader += tmpbuf;
		}
		break;
	}
	shader += "}";
    }

    // Now, the shader variable contains the string in the form we want

    shopdata = new SHOP_StringData(this);
    shopdata->setString(shader);
    return shopdata;
}

SHOP_Data *
SHOP_CustomClerk::buildGeometryData(SHOP_Node *node, float now)
{
    // If different data needs to be bound to a primitive, it's possible to
    // override this method.
    return buildRenderData(node, now);
}

const char *
SHOP_CustomClerk::getGeometryAttribute(SHOP_TYPE shader_type) const
{
    // If you want the user to be able to add a shader string on a
    // per-primitive basis (using the Shader SHOP), please return an
    // appropriate attribute name.
#if 0
    switch (shader_type)
    {
	case SHOP_SURFACE:	return "custom_surface";
	case SHOP_LIGHT:	return "custom_light";
    }
#endif
    return 0;
}

const char *
SHOP_CustomClerk::getGeometryIndirect(SHOP_TYPE shader_type) const
{
    // See the ::getGeometryAttribute method for details.
    return 0;
}

void
newShopClerk()
{
    // Here, we install the clerk for the appropriate shader types
    static SHOP_CustomClerk	*theClerk = 0;

    if (!theClerk)
    {
	theClerk = new SHOP_CustomClerk();

	SHOP_Node::addShaderClerk(SHOP_SURFACE,	theClerk);
	SHOP_Node::addShaderClerk(SHOP_LIGHT,	theClerk);
    }
}
