/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada   M5V 3E7
 *	416-504-9876
 *
 * NAME:	SHOP_CustomClerk.h ( SHOP Library, C++)
 *
 * COMMENTS:	This is a sample clerk interface to SHOPs.
 *
 * SHOPs provide a way of storing parameters to shaders.  That's all they are.
 * When a renderer requests shader information (for example, through the
 * IFD_Scene::getObjectShader(), Houdini will find a "clerk" which matches the
 * renderer type.  At that point, the clerk is called to package up a SHOP's
 * parameters in a string formatted for the particular renderer.  This clerk
 * class generates shader strings for the sample renderer in the IFD example
 * code.
 *
 * Once the clerk is in place, it's possible to add a SHOP definition which is
 * then available inside Houdini.  See the sample_install.sh script for details.
 *
 * If everything works, you should be able to compile the IFD_CustomRender and
 * SHOP_CustomClerk:
 *	Apply a cr_plastic SHOP to an object
 *	Put down an "Object Scene" output driver
 *	Choose "Custom Renderer" as the target
 *	Generate a script file
 *	Hit the render button
 * At this point, the custom clerk should be invoked and if you examine the
 * script generated, you should see a "surface" line which has the embedded
 * parameters of the SHOP enclosed in curly braces.  Something like:
 *
 *   surface cr_plastic diffuse{ 0.5 0.5 0.5} specular{ 0.5 0.5 0.5} \
 *                      ambient{ 0.5 0.5 0.5} reflect{ 0.5 0.5 0.5} phong{ 20}
 */

#ifndef __SHOP_CustomClerk__
#define __SHOP_CustomClerk__

#include <SHOP/SHOP_Clerk.h>

class SHOP_CustomClerk : public SHOP_Clerk {
public:
	     SHOP_CustomClerk();
    virtual ~SHOP_CustomClerk();

    virtual bool	 getIsVexClerk() const	{ return false; }

    virtual const char	*getRenderName() const;
    virtual const char	*getRenderLabel() const;

    virtual int		 generatesString() const;
    virtual unsigned	 disableParms(SHOP_Node &node);

    virtual SHOP_Data	*buildRenderData(SHOP_Node *node, float now);
    virtual SHOP_Data	*buildGeometryData(SHOP_Node *node, float now);

    virtual const char	*getGeometryAttribute(SHOP_TYPE shader_type) const;
    virtual const char	*getGeometryIndirect(SHOP_TYPE shader_type) const;
};

#endif
