/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *	Mark Elendt
 *	Side Effects Software Inc
 *	477 Richmond Street West
 *	Toronto, Ontario
 *	Canada   M5V 3E7
 *	416-504-9876
 *
 * NAME:	VRAY_DemoBox.h ( VRAY Library, C++)
 *
 * COMMENTS:	This is a sample procedural DSO
 */

#include <UT/UT_Args.h>
#include <GU/GU_Detail.h>
#include "VRAY_DemoBox.h"

VRAY_Procedural *
allocProcedural(const char *)
{
    return new VRAY_DemoBox();
}

VRAY_DemoBox::VRAY_DemoBox()
{
    myBox.initBounds(0, 0, 0);
}

VRAY_DemoBox::~VRAY_DemoBox()
{
}

const char *
VRAY_DemoBox::getClassName()
{
    return "VRAY_DemoBox";
}

int
VRAY_DemoBox::initialize(int argc, char *argv[], const UT_BoundingBox *)
{
    UT_Args	 args;

    // The UT_Args class is just used for convenience.  It's possible to do
    // your own argument parsing.
    args.initialize(argc, argv);
    args.stripOptions("m:::M:::");

    if (args.found('m') && args.found('M'))
    {
	myBox.initBounds(args.fargp('m', 0),
			 args.fargp('m', 1),
			 args.fargp('m', 2));
	myBox.enlargeBounds(args.fargp('m', 0),
			    args.fargp('m', 1),
			    args.fargp('m', 2));
    }
    else if (args.argc() == 7)
    {
	myBox.initBounds(atof(args(1)), atof(args(2)), atof(args(3)));
	myBox.enlargeBounds(atof(args(4)), atof(args(5)), atof(args(6)));
    }
    else
    {
	myBox.initBounds(-1, -1, -1);
	myBox.enlargeBounds( 1,  1,  1);
    }
    return 1;
}

void
VRAY_DemoBox::getBoundingBox(UT_BoundingBox &box)
{
    box = myBox;
}

void
VRAY_DemoBox::render(float /*lod*/)
{
    GU_Detail	*gdp;

    // Build some geometry
    gdp = allocateGeometry();
    gdp->cube(myBox.vals[0][0], myBox.vals[0][1],
	      myBox.vals[1][0], myBox.vals[1][1],
	      myBox.vals[2][0], myBox.vals[2][1]);

    // Now, add the geometry to mantra.
    openGeometryObject();
	addGeometry(gdp, 0);
	setSurface("showst type st");
    closeObject();
}
