/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Side Effects Software Inc., and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *      Side Effects
 *      477 Richmond Street West
 *      Toronto, Ontario
 *      Canada   M5V 3E7
 *      416-504-9876
 */

#include <string.h>
#include <malloc.h>
#include <math.h>
#include <UT/UT_DSOVersion.h>
#include <EXPR/EXPR.h>
#include <OP/OP_Director.h>
#include <CH/CH_Support.h>
#include <CH/CH_Collection.h>
#include <CMD/CMD_Manager.h>


static void
getTimeValues(float &t, float &t0, float &t1, float &v0, float &v1,
		float *m0=0, float *m1=0, float *a0=0, float *a1=0)
{
    CH_Collection	*channels;
    channels = OPgetDirector()->getChannelManager()->getEvalCollection();

    // Get the current time and the values for the function
    channels->getGlueTime(t, t0, t1, v0, v1);

    // Now, get the slope & accell
    if (m0 || m1 || a0 || a1)
	channels->getGlueSlope(m0, m1, a0, a1);
}

static void
fn_myLinear(EV_FUNCTION *, EV_SYMBOL *result, EV_SYMBOL **, int)
{
    float	t, t0, t1, v0, v1;
    float	dt;

    getTimeValues(t, t0, t1, v0, v1);
    dt = t1 - t0;
    result->value.fval = (dt == 0) ? (v0+v1)*0.5F
				   : v0 + (v1-v0)*(t - t0)/dt;
}

#define EVF	EV_TYPEFLOAT
#define EVS	EV_TYPESTRING
#define EVV	EV_TYPEVECTOR

//
//  These flags are important so that JIVE will display the correct handles for
//  the curve.
#define VAL_FLAGS	(CH_EXPRVALUE)
#define VALSLOPE_FLAGS	(CH_EXPRVALUE | CH_EXPRSLOPE)

static EV_FUNCTION funcTable[] = {
    EV_FUNCTION(VAL_FLAGS, "myLinear",	0, EVF,	0,	fn_myLinear),
    EV_FUNCTION(),
};

void
CMDextendLibrary(CMD_Manager *)
{
    int		i;

    for (i = 0; funcTable[i].getName(); i++)
	ev_AddFunction(&funcTable[i]);
}
